@@ -15,6 +15,7 @@ class EsriDumper(object):
                  outSR=None, proxy=None,
                  start_with=None, geometry_precision=None,
                  paginate_oid=False,
+                 max_page_size=None,
                  pause_seconds=10, requests_to_pause=5, num_of_retry=5):
         self._layer_url = url
         self._query_params = extra_query_args or {}
@@ -27,6 +28,7 @@ class EsriDumper(object):
         self._startWith = start_with or 0
         self._precision = geometry_precision or 7
         self._paginate_oid = paginate_oid
+        self._max_page_size = max_page_size or 1000
 
         self._pause_seconds = pause_seconds
         self._requests_to_pause = requests_to_pause
@@ -157,7 +159,7 @@ class EsriDumper(object):
         response = self._request('GET', url, params=query_args, headers=headers)
         count_json = self._handle_esri_errors(response, "Could not retrieve row count")
         count = count_json.get('count')
-        if not count:
+        if count is None:
             raise EsriDownloadError("Server doesn't support returnCountOnly")
         return count_json['count']
 
@@ -300,7 +302,7 @@ class EsriDumper(object):
     def __iter__(self):
         query_fields = self._fields
         metadata = self.get_metadata()
-        page_size = min(1000, metadata.get('maxRecordCount', 500))
+        page_size = min(self._max_page_size, metadata.get('maxRecordCount', 500))
         geometry_type = metadata.get('geometryType')
 
         row_count = None
@@ -310,6 +312,11 @@ class EsriDumper(object):
         except EsriDownloadError:
             self._logger.info("Source does not support feature count")
 
+        # If there are no records matching the query, short circuit and return an empty generator.
+        if row_count == 0:
+            return
+            yield
+
         page_args = []
 
         if not self._paginate_oid and row_count is not None and (metadata.get('supportsPagination') or \
